# ******************************************************
# *
# * Name:         step01-create-blob-storage.ps1
# *     
# * Design Phase:
# *     Author:   John Miner
# *     Date:     01-21-2018
# *     Purpose:  Create two containers and load sample
# *               stock data.  Create a shared 
# *               access signature for use later.
# * 
# ******************************************************


#
# Azure Subscriptions 
#

# Import security credentials
Import-AzureRmContext-Path"C:\COMMUNITY WORK\STAGING\ATS\AZURE-CREDS.JSON" 

# List my subscriptions
Clear-Host
Get-AzureRmSubscription




#
# Create a resource group
#

# New resource group
New-AzureRmResourceGroup -Name "rg4tips18" -Location "East US 2"

# List resource groups
Clear-Host
Get-AzureRmResourceGroup -Name "rg4tips18"

# Delete resource group
# Remove-AzureRmResourceGroup -Name "rg4tips18" -Force



#
# Create a storage account
#

# Create new storage account (lowercase)
New-AzureRmStorageAccount StorageAccountName 'sa4tips18' -ResourceGroupName "rg4tips18" -Location "East US 2" -Type "Standard_LRS"

# Clear the screen
Clear-Host
Get-AzureRmStorageAccount | Select-Object ResourceGroupName, StorageAccountName, Location, Kind, CreateTime, Encryption


# Show the account
$A = Get-AzureRmStorageAccount 
$A.Sku
$A.Type

# Delete storage account
#Remove-AzureRmStorageAccount -ResourceGroupName "rg4tips18" -Name 'sa4tips18' 



#
# Create a storage container(s)
#

# Grab storage context - work around for RM
$StorageContext = Get-AzureRmStorageAccount -Name 'sa4tips18' -ResourceGroupName "rg4tips18" 

# Create new containers
$StorageContext | New-AzureStorageContainer -Name "sc4inbound" -Permission Off
$StorageContext | New-AzureStorageContainer -Name 'sc4archive' -Permission Off

# Clear the screen
Clear-Host

# Show the container
Clear-Host
$StorageContext | Get-AzureStorageContainer 
 
# Remove the container
# $StorageContext | Remove-AzureStorageContainer -Name "sc4inbound" -Force
# $StorageContext | Remove-AzureStorageContainer -Name 'sc4archive' -Force



#
# Upload files to azure (stocks)
#

# Grab storage context - work around for RM
$StorageContext = Get-AzureRmStorageAccount -Name 'sa4tips18' -ResourceGroupName "rg4tips18" 

# Set source path
$srcpath = "C:\COMMUNITY WORK\STAGING\DATA\S&P-2017\"

# Set counter
$cnt = 0

# Local file path
$list = Get-ChildItem $srcpath -Filter *.csv 

# For each stock
foreach ($file in $list)
{
    # file names
    $localname = $srcpath + $file.Name
    $remotename = "NEW/" + $file.name

    # Upload file to azure
    $StorageContext | Set-AzureStorageBlobContent -Container "sc4inbound" -File $localname -Blob $remotename -Force

    # Increment count
    Write-Host $srcfull
    Write-Host $cnt
    $cnt = $cnt + 1
}



#
# Upload master stock list
#

# Grab storage context - work around for RM
$StorageContext = Get-AzureRmStorageAccount -Name 'sa4tips18' -ResourceGroupName "rg4tips18" 

# Set source path
$srcpath = "C:\COMMUNITY WORK\STAGING\DATA\S&P-2017\"
$filename = "PACKING-LIST.TXT"

# File names
$localname = $srcpath + $filename
$remotename = "NEW/" + $filename

# Upload file to azure
$StorageContext | Set-AzureStorageBlobContent -Container "sc4inbound" -File $localname -Blob $remotename -Force



#
# Upload file summary
#

# Grab storage context - work around for RM
$StorageContext = Get-AzureRmStorageAccount -Name 'sa4tips18' -ResourceGroupName "rg4tips18" 

# Set source path
$srcpath = "C:\COMMUNITY WORK\STAGING\DATA\S&P-2017\"
$filename = "PACKING-SUM.TXT"

# File names
$localname = $srcpath + $filename
$remotename = "NEW/" + $filename

# Upload file to azure
$StorageContext | Set-AzureStorageBlobContent -Container "sc4inbound" -File $localname -Blob $remotename -Force




#
# List all blobs in the container
#


# List all blobs in a container.
$StorageContext | Get-AzureStorageBlob -Container "sc4inbound" 



#
# Download file from azure
#

$StorageContext | Get-AzureStorageBlob -Container "sc4inbound" -Blob "PACKING-LIST.TXT" | Get-AzureStorageBlobContent -Destination "c:\temp\"



# https://docs.microsoft.com/en-us/powershell/module/azurerm.storage/get-azurermstorageaccountkey?view=azurermps-5.1.1

#
# Grab storage key for azure blob storage
#

# Clear the screen
Clear-Host
Write-Host " "
Write-Host " "

# Storage key 1
$Key1 = (Get-AzureRmStorageAccountKey -ResourceGroupName "rg4tips18" -AccountName "sa4tips18").Value[0]
Write-Host "Storage Key 1 = $Key1"
Write-Host " "

# Storage key 1
$Key2 = (Get-AzureRmStorageAccountKey -ResourceGroupName "rg4tips18" -AccountName "sa4tips18").Value[1]
Write-Host "Storage Key 2 = $Key2"
Write-Host " "
Write-Host " "


<#
 
Storage Key 1 = XyhfR0Rc5mRFoZl/EGZuDPaUQ61zNtG9z4Jtu2N8vaQ83kYBagsIxHO5b1D9RrKlcWs2h6/rZc5DbApXVDSXhw==
 
Storage Key 2 = kpYhyYpVu9Y2ExTKr+8wsTy9H3UXQZqp4oJqsR4zIs+avoHBF9E6dkIWkFtMh9o8piJOOEIRdiNdY6sRcnrC3A==

#>



#
# Create Shared Access Signature (SAS)
#

# Grab storage context - work around for RM
$StorageContext = Get-AzureRmStorageAccount -Name "sa4tips18" -ResourceGroupName "rg4tips18" 

# Sets up a Stored Access Policy and a Shared Access Signature for the new container  
$Policy = $StorageContext | New-AzureStorageContainerStoredAccessPolicy -Container "sc4inbound" -Policy "pol4inbox" `
  -StartTime $(Get-Date).ToUniversalTime().AddMinutes(-5) -ExpiryTime $(Get-Date).ToUniversalTime().AddYears(10) -Permission rwld

# Gets the Shared Access Signature for the policy  
$SAS = $StorageContext | New-AzureStorageContainerSASToken -name "sc4inbound" -Policy "pol4inbox" 
Write-Host 'Shared Access Signature= '$($SAS.Substring(1))''  

<#

 sv=2017-04-17&sr=c&si=pol4inbox&sig=CuINwl6%2Falc6jEvUFI1yTV4fUVEZmCuBVr%2Fr%2BSQWRbc%3D 

#>