# ******************************************************
# *
# * Name:         step02-create-azure-sql-dw.ps1
# *     
# * Design Phase:
# *     Author:   John Miner
# *     Date:     01-21-2018
# *     Purpose:  Create azure sql database dw 
# *               using resource manager.
# * 
# ******************************************************


#
# Azure Subscriptions 
#

# Import security credentials
Import-AzureRmContext-Path"C:\COMMUNITY WORK\STAGING\ATS\AZURE-CREDS.JSON" 

# List my subscriptions
Clear-Host
Get-AzureRmSubscription



#
# Data Centers with my service
#

# Clear the screen
Clear-Host

# Data centers with Azure SQL database
$AzureSQLLocations = (Get-AzureRmResourceProvider -ListAvailable | 
    Where-Object {$_.ProviderNamespace -eq 'Microsoft.Sql'}).Locations
$AzureSQLLocations




#
# Create a resource group
#

# New resource group
New-AzureRmResourceGroup -Name "rg4tips18" -Location "East US 2"

# Clear the screen
Clear-Host

# List resource groups
Get-AzureRmResourceGroup -Name "rg4tips18"

# Delete resource group
# Remove-AzureRmResourceGroup -Name "rg4tips18" -Force



#
# Create a new server
#

# Create a credential
$User = "jminer"
$Pword = ConvertTo-SecureString String 'MS#Sql4tips$2018' AsPlainText -Force
$Credential = New-Object TypeName System.Management.Automation.PSCredential ArgumentList $User, $Pword

# New sql server
New-AzureRmSqlServer -ResourceGroupName "rg4tips18" -ServerName "sql4tips18" -Location "East US" -SqlAdministratorCredentials $Credential

# Clear the screen
Clear-Host

# List Sql Servers
Get-AzureRmSqlServer -ResourceGroupName "rg4tips18"

# Remove the sql server
#Remove-AzureRmSqlServer -ResourceGroupName "rg4tips18" -ServerName "sql4tips18" 



#
# Add laptop ip to firewall (todo = code to figure out ip since I am behind a router)
#

# External ip
$MyIp = (Invoke-WebRequest ifconfig.me/ip).Content -replace "`n","" 

# Create new firewall rule
New-AzureRmSqlServerFirewallRule -ResourceGroupName "rg4tips18" -ServerName "sql4tips18" -FirewallRuleName "fr4laptop" -StartIpAddress "$MyIp" -EndIpAddress "$MyIp"

# Clear the screen
Clear-Host

# List firewall rules
Get-AzureRmSqlServerFirewallRule -ResourceGroupName "rg4tips18" -ServerName "sql4tips18"


# Remove firewall rule
# Remove-AzureRmSqlServerFirewallRule -ResourceGroupName "rg4tips18" -ServerName "sql4tips18" -FirewallRuleName "fr4laptop"



# https://docs.microsoft.com/en-us/azure/sql-data-warehouse/performance-tiers

# -- New tiers ~ optimize for Elasticity vs Compute

# https://azure.microsoft.com/en-us/blog/faster-compute-optimized-performance-tier-for-azure-sql-data-warehouse/


#
# Make the new database
#

# Give database size quota
$TenTb = (1024 * 1024 * 1024 * 1024 * 10)
$TenTb

# Create a new database
New-AzureRmSqlDatabase -RequestedServiceObjectiveName "DW1000" -DatabaseName "dw4stocks" `
    -ServerName "sql4tips18" -ResourceGroupName "rg4tips18" -Edition "DataWarehouse" `
    -CollationName "SQL_Latin1_General_CP1_CI_AS" -MaxSizeBytes $TenTb

# Clear the screen
Clear-Host

# List the new database
Get-AzureRmSqlDatabase -ResourceGroupName "rg4tips18" -ServerName "dw4stocks"


# Remove the database
# Remove-AzureRmSqlDatabase -ResourceGroupName "rg4tips18" -ServerName "sql4tips18" -DatabaseName "dw4stocks"



#
# Change the database size (DW100)
#

# Log into Azure
Login-AzureRmAccount

# Clear the screen
Clear-Host

# Give database size quota
Set-AzureRmSqlDatabase -ResourceGroupName "rg4tips18" -ServerName "sql4tips18"  `
  -DatabaseName "dw4stocks" -Edition "DataWarehouse" -RequestedServiceObjectiveName "DW100"



#
# Pause the compute charges
#


$dbinfo = Get-AzureRmSqlDatabase ResourceGroupName "rg4tips18" ServerName "sql4tips18" DatabaseName "dw4stocks"
$result = $dbinfo | Suspend-AzureRmSqlDatabase
$result



#
# Resume the compute charges
#


$dbinfo = Get-AzureRmSqlDatabase ResourceGroupName "rg4tips18" ServerName "sql4tips18" DatabaseName "dw4stocks"
$result = $dbinfo | Resume-AzureRmSqlDatabase
$result